/*
	File:		Shell.r

	Contains:	Resources for the document shell

	Owned by:	Nick Pilch

	Copyright:	 1993 - 1996 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		<23>	  10/23/96	TJ		Removed LowMemDlg.rsrc
		<22>	  10/23/96	DH		Removed outdated resources to fix problem
									with previous checkin.
		<21>	  10/22/96	DH		1336808,1333949,1368888,1375780 Included
									low-mem dialog resources.
		<20>	 10/4/96	CSL		1334277: Move AboutBox resources to
									AboutBox.r
		<19>	27.09.1996	NP		1385775: Incorrect dialog after OD
									initialization fails.
		<18>	20.09.1996	NP		1389025: OpenDoc Setup changes for new
									config.
		<17>	 9/18/96	TJ		1384389 Remove RSRC names in Non-Debug
									Builds.
		<16>	 9/10/96	eeh		1384356: make resources purgeable
		<15>	 7/30/96	eeh		1372943: new goodbye alerts
		<14>	 6/23/96	NP		10002: Launch time speedups.
		<13>	 6/22/96	TJ		Use AboutDlogs.res instead of OpenDocX.res.
		<12>	 6/21/96	jpa		1358818: Added "File" menu for container
									apps.
		<11>	 6/19/96	NP		10002: Launch time speedups.
		<10>	 6/18/96	NP		10002: Launch time speedups.
		 <9>	 5/31/96	jpa		T10012: Added dreaded Quit command
									(CyberDog only.)
		 <8>	.04.1996	NP		One more time: non-debug build fix.
		 <7>	.04.1996	NP		Oops. Didn't work for a non-debug build.
		 <6>	.04.1996	NP		1286751: Present alert when trying to use
									Open when no Finder is running.
		 <5>	 3/29/96	DM		1334273: added new alert 493 which
									announces OpenDoc must quit immediately due
									to very low memory.
		 <4>	.03.1996	NP		1307182: Removed STR# item.
		 <3>	 3/14/96	NP		1330149: Remove objectionable MENU entries.
		 <2>	 1/15/96	JA		1269485: Resized About dlog & added 'ictb'
									to change font.
		<43>	10/31/95	eeh		1297192: make room for strings in dialogs
		<42>	10/31/95	TJ		Made the Splash Screen Non-Modal.#1297413
		<41>	10/26/95	T		1296574 GM:Cannot set default memory
									partition size.
		<40>	10/25/95	T		1281409 GM: OpenDoc needs an AboutBox.
		<39>	10/24/95	jpa		1293441: Added more lo-mem alerts.
		<38>	10/18/95	CC		GM 1293627: Use
									alertPositionParentWindowScreen for all
									alerts.
		<37>	10/16/95	CC		GM 1291511: Layout changes to Save As
									dialog to hide Kind popup.
		<36>	 10/4/95	jpa		CmmnUtls.r --> UtilCmmn.r [1285767]
		<35>	 9/29/95	eeh		1287262: include CmmnUtls.r, remove quotes
		<34>	 9/27/95	eeh		1284893: fix capitalization
		<33>	 9/20/95	T		1285201 GM: Use
									alertPositionParentWindowScreen for dialogs
		<32>	 9/12/95	JBS		1282526 Status message needed during "Save
									Doc to another volume"
		<31>	 9/12/95	eeh		1253309: add kSHLPluginErrorStrID string
		<30>	  9/8/95	T		1252120 FB1: New documents saved w/ StdFile
									& trashed if closed w/o saving
		<29>	  9/8/95	eeh		1268998: add prefs file name 'STR '
									resource
		<28>	  9/7/95	eeh		1278987: 'STR ' resources for UniquifyName
		<27>	 8/22/95	eeh		1276281: template strings
		<26>	 8/15/95	T		1258489 Remove TM from OpenDoc folder
		<25>	  8/8/95	jpa		Added kSHLDragToTrashAlert (commented out)
									[1259401]
		<24>	 6/20/95	T		1217754 BB: Delete Document command
		<23>	  6/8/95	RR		#1257260 Collapse base classes. #1214898
									StdTypes.r -> ODTypes.r
		<22>	  6/1/95	jpa		Added ^2 to error DITL so we can show
									exception message str. [1242839]
		<21>	 5/19/95	T		1155852 BB: String constants need to be
									moved to resource for Localization
		<20>	  5/7/95	T		1155852 BB: String constants need to be
									moved to resource for Localization (mostly
									done)
		<19>	  5/4/95	jpa		Changed text of low-mem alert and added app
									low-mem alert [1235657]
		<18>	  5/1/95	CL		Change Revert and Save Changes ALRTs to
									DLOGs, add STR# to support cmd-keys for
									buttons [1228662,1228678,1229912].
		<17>	 4/26/95	TJ		Removed version Resources, they are now in
									ODVersion.r
		<16>	 4/13/95	CG		#1221869: Added STR resources for Shell
									Plug-Ins folder name.
		<15>	 4/10/95	T		#1236210	BB: "Show Parts Bin" menu item
									must be removed from the Edit menu
		<14>	 2/20/95	T		#1210979 BB: 5-$ Bugs need to be evaluated
									and removed from Shell
									#1221861 BB: Incomplete AOCE support needs
									to be removed from OpenDoc
		<13>	 2/16/95	CG		#1191912: Changed Print to Print
		<12>	  1/6/95	jpa		Read DocStub.rsrc, not DocStub.r.
		<11>	12/25/94	T		1191189 Eliminate duplicate Def.h, Def.xh
									files
									1187709 More frequent, robust and earlier
									error detection and reporting in the Shell
									Moved Error message resources to ErrMsgs.r.
		<10>	11/17/94	CG		(for TC) 1194570, 1193996, 1194812, 1191912
									- user experience fixes.
		 <9>	 10/7/94	T		#1187826 Document Info functionality
		 <8>	 9/19/94	T		#1150345 Shell.r does not need to refer to
									Test Parts
		 <7>	 9/16/94	eeh		#1186754: eliminate non-AOCE document menu.
		 <6>	 8/25/94	T		#1183223 Enabled launching of documents in
									folders other than that of the OpenDoc
									libraries.  Added document stub resources.
		 <5>	 8/19/94	T		#1159233 Remove references to j3di
		 <4>	  8/8/94	jpa		Added "Heap Info" dialog [1179567]
		 <3>	 7/21/94	T		#define oldTemp // to get 'dctb' resources
									to rez
		 <2>	 7/21/94	eeh		fix to comments
		 <1>	  7/6/94	T		first checked in
		 <0>	  7/6/94	SV		SOMverted
		<39>	 4/11/94	T		#1155854 Use CustomPutFile for Save A Copy.
		<38>	  4/7/94	T		fixed #1155956
		<37>	 3/23/94	CG		#1151393: Use OpenDoc-wide consts for
									'vers' 1 and 2 resources.
		<36>	  3/9/94	eeh		bug #1149630: reversed "Forward" and
									"Reply" in mailer menu.
		<35>	 2/15/94	T		fix #1144029 and some cleanup
		<34>	 1/24/94	NP		Changed DITL for About Box to enable the
									PICT item.
		<33>	 1/19/94	eeh		added Open Document menu item
		<32>	 1/18/94	T		removed scsz resource for AppleScript, the
									Shell library does not need an scsz
									resource, only the OpenDoc extension and
									documents do.
		<31>	 1/18/94	T		use kOpenDocShellSignature & remove BNDL &
									FREF resources (why were they there?)
		<30>	 1/13/94	eeh		fixed name of document menu
		<29>	 1/11/94	eeh		make mailer menu heierarchical
		<28>	12/17/93	eeh		changed dontReadExtensionTerms to
									readExtensionTerms in 'scsz'
		<27>	 12/9/93	eeh		changed rezid of 'scsz' to 0
		<26>	 12/3/93	T		Rename ODError.h to ErrorDef.h
		<25>	11/30/93	NP		Added alwaysSendSubject to scsz resource.
		<24>	11/24/93	T		remove comments about SIZE -1 resource
		<23>	11/15/93	eeh		removed 'aete' resource; added menus for
									Reply and Forward (mailer)
		<22>	11/12/93	CG		Added scsz resource for AppleScript.
		<21>	10/27/93	T		update type 'errs' for the change of
									ODError from ODULong to ODSLong
		<20>	10/20/93	eeh		Readded expand- and contract-mailer menus
									(for use during development only).
		<19>	10/18/93	T		moved SIZE -1 resource to Document.r
		<18>	10/12/93	eeh		changes to AOCE menu
		<17>	 10/8/93	T		bifurcated Document menu into normal and
									AOCE versions
		<16>	 10/7/93	VL		Added more error codes.
		<15>	 10/7/93	eeh		various changes related to adding Mailer
									support (more to come)
		<14>	 10/1/93	RR		Fix file corruption
		<13>	 10/1/93	T		fixed definition of 'errs' resource.
									UNSIGNED LONGINT for fields
		<12>	 9/30/93	T		changed 'errs' resource to use LONGINTs for
									error numbers and ranges
		<11>	 9/22/93	NP		Changed aete resource.
		<10>	 9/17/93	T		changed 'File' menu to 'Document'
		 <9>	 9/15/93	T		added Error Handling resources, 'errs' etc.
		 <8>	 9/10/93	RR		Remove File and Edit from MBAR resource
		 <7>	 8/27/93	T		added About Box from Prototype
		 <6>	 8/19/93	T		updated Menus for HI Changes
		 <5>	  8/4/93	T		added AETE resource
		 <4>	 7/21/93	T		updated standard DocShell menus
		 <3>	 5/24/93	T		add Save Changes alert (stolen from
									TeachText) and Revert alert
		 <2>	 4/30/93	T		isStationeryAware
		 <1>	 4/27/93	RCR		First Checked In.

	To Do:
	In Progress:
		
*/

#ifndef _SHELLDEF_
#include "ShellDef.h"
#endif

#ifndef SOM_Module_OpenDoc_StdDefs_defined
#include "StdDefs.r"
#endif


#define SystemSevenOrBetter 1			// we want the extended types
#define	SystemSevenOrLater	1			// Types.r uses this variable

#define oldTemp			// to get 'dctb' resources to rez
#ifndef __TYPES_R__
#include "Types.r"
#endif
#undef oldTemp

#ifndef __SYSTYPES_R__
#include "SysTypes.r"
#endif

#ifndef _CONSTDEF_
#include "ConstDef.h"
#endif

#ifndef _UTILDEFS_
#include "UtilDefs.h"
#endif

include "DocStub.rsrc";					// Include document-stub resources


#define	rMenuBar	128				/* application's menu bar */
#define	rAboutAlert	128				/* about alert */
#define	rDocWindow	128				/* application's window */

/* The following constants are used to identify menus and their items. The menu IDs
   have an "m" prefix and the item numbers within each menu have an "i" prefix. */
#define	mApple					255		/* Apple menu */
#define	iAbout					1

#define	mFile					256		/* File menu */

#define	mEdit					257		/* Edit menu */


resource 'MBAR' (255, preload) {
	{ mApple  };		/* File and edit are added later */
};


resource 'MENU' (mApple, preload) {
	mApple, textMenuProc,
	0b1111111111111111111111111111101,	/* disable dashed line, enable About and DAs */
	enabled, apple,
	{
		"About OpenDoc",
			noicon, nokey, nomark, plain;
		"-",
			noicon, nokey, nomark, plain
	}
};

resource 'MENU' (mFile) {
	/* Note: When changing this menu, keep the "File" menu below in sync! */
	mFile, textMenuProc,
	0b0000000000000010001011111010101,	
	enabled, "Document",
	{
		"New",
			noicon, "N", nomark, plain;
		"Open Selection",
			noicon, nokey, nomark, plain;
		"Open Document",
			noicon, "O", nomark, plain;
		"Insert",
			noicon, nokey, nomark, plain;
		"Close",
			noicon, "W", nomark, plain;
		"Delete",
			noicon, nokey, nomark, plain;
		"-",
			noicon, nokey, nomark, plain;
		"Save",
			noicon, "S", nomark, plain;
		"Save a Copy",
			noicon, nokey, nomark, plain;
		"Revert to Saved",
			noicon, nokey, nomark, plain;
		"Drafts",
			noicon, nokey, nomark, plain;
		"Document Info",
			noicon, nokey, nomark, plain;
		"-",
			noicon, nokey, nomark, plain;
		"Page Setup",
			noicon, nokey, nomark, plain;
		"Print",
			noicon, "P", nomark, plain;
	}
};

resource 'MENU' (kODFileMenuID) {
	mFile, textMenuProc,	/* same menuID as regular Document menu */
	0b0000000000000010001011111010101,	
	enabled, "File",		/* ...but different title */
	{
		"New",
			noicon, "N", nomark, plain;
		"Open Selection",
			noicon, nokey, nomark, plain;
		"Open Document",
			noicon, "O", nomark, plain;
		"Insert",
			noicon, nokey, nomark, plain;
		"Close",
			noicon, "W", nomark, plain;
		"Delete",
			noicon, nokey, nomark, plain;
		"-",
			noicon, nokey, nomark, plain;
		"Save",
			noicon, "S", nomark, plain;
		"Save a Copy",
			noicon, nokey, nomark, plain;
		"Revert to Saved",
			noicon, nokey, nomark, plain;
		"Drafts",
			noicon, nokey, nomark, plain;
		"Document Info",
			noicon, nokey, nomark, plain;
		"-",
			noicon, nokey, nomark, plain;
		"Page Setup",
			noicon, nokey, nomark, plain;
		"Print",
			noicon, "P", nomark, plain;
		"-",
			noicon, nokey, nomark, plain;
		"Quit",					/* ...and a Quit command */
			noicon, "Q", nomark, plain;
	}
};

resource 'MENU' (mEdit, preload) {
	mEdit, textMenuProc,
	0b0000000000000000000000000000000,	/* disable everything, program does the enabling */
	enabled, "Edit",
	 {
		"Undo",
			noicon, "Z", nomark, plain;
		"Redo",
			noicon, "R", nomark, plain;
		"-",
			noicon, nokey, nomark, plain;
		"Cut",
			noicon, "X", nomark, plain;
		"Copy",
			noicon, "C", nomark, plain;
		"Paste",
			noicon, "V", nomark, plain;
		"Paste As",
			noicon, nokey, nomark, plain;
		"Clear",
			noicon, nokey, nomark, plain;
		"Select All",
			noicon, "A", nomark, plain;
		"-",
			noicon, nokey, nomark, plain;
		"Part Info",
			noicon, "L", nomark, plain;
		"Preferences",
			noicon, nokey, nomark, plain;
		"-",
			noicon, nokey, nomark, plain;
		"View in Window",
			noicon, nokey, nomark, plain
	}
};

/* saving to a different volume status dialog */
	/* $$$$$ fix coordinates below */

resource 'DLOG' (kSHLSaveDiffVolDialogID, purgeable /*<eeh> added for 1384356*/) {
	{40, 40, 92, 297},
	dBoxProc,
	visible,
	goAway,
	0x0,
	kSHLSaveDiffVolDialogID,
	"",
	alertPositionParentWindowScreen
};

resource 'DITL' (kSHLSaveDiffVolDialogID, purgeable /*<eeh> added for 1384356*/) {
	{	/* array DITLarray: 2 elements */
		/* [1] */
		{10, 20, 42, 52},
		Icon {
			disabled,
			1
		},
		/* [2] */
		{10, 65, 42, 259},
		StaticText {
			disabled,
			"Saving Document..."	/* $$$$$ should have document name */
		}
	}
};


/* "Do you want to save changes?" dialog */
resource 'DLOG' (kSHLsvChanges, purgeable /*<eeh> added for 1384356*/) {
	{94, 80, 211, 434},	
	dBoxProc, invisible, noGoAway, 0,
	kSHLsvChanges,
	"",
	alertPositionParentWindowScreen
};

/* Item list for "Do you want to save changes?" dialog 			 */
/* Note: Strings in the STR# following this DITL correspond to	 */
/* Each DITL item.  Strings represent the command key code       */
/* for buttons in this DITL.									 */
resource 'DITL' (kSHLsvChanges, purgeable /*<eeh> added for 1384356*/) {
	{	/* array DITLarray: 5 elements */
		/* [1] */
		{87, 284, 107, 344},
		Button {
			enabled,
			"Save"
		},
		/* [2] */
		{87, 211, 107, 271},
		Button {
			enabled,
			"Cancel"
		},
		/* [3] */
		{87, 70, 107, 155},
		Button {
			enabled,
			"Dont Save"
		},
		/* [4] */
		{10, 20, 42, 52},
		Icon {
			disabled,
			2
		},
		/* [5] */
		{10, 72, 76, 344},
		StaticText {
			disabled,
			"Save changes to the document ^0 before closing?"
		}
	}
};


/* Command-key codes for corresponding dialog items in Save Changes */
resource 'STR#' ( kSHLsvChangesCmdKeyStrs, purgeable ) {
	{	"s",	/* Save button 		 */
		".",	/* Cancel button 	 */
		"d"		/* Don't Save button */	}
};


/* Document-dragged-to-Trash alert: */

/* Currently no alert is used. If it will be, uncomment this resource. */
#if 0
	resource 'ALRT' (kSHLDragToTrashAlert 
	#If ODDebug
	,"Trash Alert"
	#endif
	, purgeable) {
		{40, 40, 210, 430},
		kSHLDragToTrashAlert,
		beepStages, 
		alertPositionParentWindowScreen
	};

	resource 'DITL' (kSHLDragToTrashAlert,
#if ODDebug
"Trash Alert",
#endif
purgeable /*<eeh> added for 1384356*/) {
		{
			{140, 310, 160, 370},	Button { enabled, "Save" },
			{140, 70, 160, 150},	Button { enabled, "Don't Save" },
			{140, 160, 160, 300},	Button { enabled, "Remove From Trash" },
			{10, 20, 42, 52},		Icon   { disabled, 2 },
			{10, 70, 130, 370},		StaticText { disabled,
				"The document ^0 has been dragged to the trash. Changes have been made "
				"since the last time this document was saved. \n\nSave changes to this "
				"document before closing it?"
			}
		}
	};
#endif


/* "Revert to the last saved version?" dialog */
resource 'DLOG' (kSHLsvRevert, purgeable /*<eeh> added for 1384356*/) {
	{94, 80, 194, 380},
	dBoxProc, invisible, noGoAway, 0,
	kSHLsvRevert,
	"",
	alertPositionParentWindowScreen
};

/* Item list for Revert dialog 									 */
/* Note: Strings in the STR# following this DITL correspond to	 */
/* Each DITL item.  Strings represent the command key code       */
/* for buttons in this DITL.									 */
resource 'DITL' (kSHLsvRevert, purgeable /*<eeh> added for 1384356*/) {
	{	/* array DITLarray: 4 elements */
		/* [1] */
		{73, 230, 93, 290},
		Button {
			enabled,
			"Revert"
		},
		/* [2] */
		{73, 157, 93, 217},
		Button {
			enabled,
			"Cancel"
		},
		/* [3] */
		{10, 20, 42, 52},
		Icon {
			disabled,
			2
		},
		/* [4] */
		{10, 71, 61, 300},
		StaticText {
			disabled,
			"Revert to the last saved version of the document ^1?"
		}
	}
};


/* Command-key codes for corresponding dialog items in Revert Dialog */
resource 'STR#' ( kSHLsvRevertCmdKeyStrs, purgeable ) {
	{	"r",	/* Revert button */
		"."		/* Cancel button */	}
};

/* "Delete this document?" dialog */
resource 'DLOG' (kSHLDeleteDLOG, purgeable /*<eeh> added for 1384356*/) {
	{94, 80, 194, 380},
	dBoxProc, invisible, noGoAway, 0,
	kSHLDeleteDLOG,
	"",
	alertPositionParentWindowScreen
};

/* Item list for Delete dialog 									 */
/* Note: Strings in the STR# following this DITL correspond to	 */
/* Each DITL item.  Strings represent the command key code       */
/* for buttons in this DITL.									 */
resource 'DITL' (kSHLDeleteDLOG, purgeable /*<eeh> added for 1384356*/) {
	{	/* array DITLarray: 4 elements */
		/* [1] */
		{73, 230, 93, 290},
		Button {
			enabled,
			"Delete"
		},
		/* [2] */
		{73, 157, 93, 217},
		Button {
			enabled,
			"Cancel"
		},
		/* [3] */
		{10, 20, 42, 52},
		Icon {
			disabled,
			2
		},
		/* [4] */
		{10, 71, 61, 300},
		StaticText {
			disabled,
			"Delete the document ^1?"
		}
	}
};


/* Command-key codes for corresponding dialog items in Revert Dialog */
resource 'STR#' ( kSHLDeleteDLOGCmdKeyStrs, purgeable ) {
	{	"d",	/* Delete button */
		"."		/* Cancel button */	}
};


/* Save a copy dialog */
resource 'DLOG' (kSHLSaveCopyDlgID
#If ODDebug
, "Save A Copy"
#endif
, purgeable) {
	{0, 0, 178+11+19+7, 344},
	dBoxProc, invisible, noGoAway, 0,
	kSHLSaveCopyDlgID, 
	"",
	alertPositionParentWindowScreen
};

resource 'DITL'(kSHLSaveCopyDlgID,
#if ODDebug
"Save A Copy",
#endif
purgeable /*<eeh> added for 1384356*/) {	
	{
		/*   The first 12 items are from the default Save dialog
		 *   as described on p.age 3-18 and 3-19 of Inside Mac:Files
		 */ 
		/* sfItemOpenButton */
		{154+16+12, 252, 174+16+12, 332}, Button { enabled, "Save" },
		/* sfItemCancelButton */
		{123+12+12, 252, 143+12+12, 332}, Button { enabled, "Cancel" },
		/* sfItemBalloonHelp */
		{0, 0, 0, 0}, UserItem { disabled }, /* HelpItem { disabled, HMScanhdlg {-6043}},*/
		/* sfItemVolumeUser */
		{8, 235, 24, 337}, UserItem { enabled },
		/* sfItemEjectButton */
		{32, 252, 52, 332}, Button { enabled, "Eject" },
		/* sfItemDesktopButton */
		{60, 252, 80, 332}, Button { enabled, "Desktop" },
		/* sfItemFileListUser */
		{29, 12, 127, 230}, UserItem { enabled },
		/* sfItemPopUpMenuUser */
		{6, 12, 25, 230}, UserItem { enabled },
		/* sfItemDividerLinePict */
		{115+6+6, 250, 116+6+6, 334}, Picture { disabled, 11 },
		/* sfItemFileNameTextEdit */
		{150, 15, 150+16, 227}, EditText { enabled, "" },
		/* sfItemPromptStaticText */
		{129, 15, 129+16, 227}, StaticText { disabled, "Save as:" },
		/* sfItemNewFolderUser */
		{88, 252, 108, 332}, UserItem { disabled },
		/* 
		 *   New items specific to OpenDoc follow
		 */
		/* kSHLSaCDocumentRadioBtn */
		{166+18, 15, 166+18+16, 120}, RadioButton {enabled, "    Document"},
		/* kSHLSaCStationeryRadioBtn */
		{166+18, 125, 166+18+16, 227}, RadioButton {enabled, "    Stationery"},
		/* kSHLSaCKindPopupItem */
		{166+11+16+8+1000, 15-3, 166+11+16+8+19+1000, 227+3}, Control {disabled, kSHLSaCKindPopupItem},
		/* kSHLSaCDocumentIconItem */
		{166+17, 15+17, 166+17+16, 15+17+16}, UserItem {disabled},
		/* kSHLSaCStationeryIconItem */
		{166+17, 125+17, 166+17+16, 125+17+16}, UserItem {disabled}
	}
};


/* Dialog color table -- allows CustomPutFile to use color */
resource 'dctb' (kSHLSaveCopyDlgID,
#if ODDebug
"Save a Copy",
#endif
purgeable /*<eeh> added for 1384356*/) {
	0,
	0,
	{
		wContentColor,0xFFFF,0xFFFF,0xFFFF
	}
};


resource 'CNTL' (kSHLSaCKindPopupItem) {
		{173+11, 15-3, 173+11+19, 227+3},
		popupTitleRightJust,
		visible,
		40,					/* width of title in pixels     */
		kSHLSaCKindPopupMenu,				/* resource id of MENU resource */
		popupMenuCDEFproc+popupUseWFont+popupFixedWidth,
		0,					/* Refcon */
		"Kind:"
};

resource 'MENU' (kSHLSaCKindPopupMenu
#If ODDebug
, "Kind"
#endif
) {
	kSHLSaCKindPopupMenu,
	textMenuProc,
	allEnabled,
	enabled,
	"Kind",
	{	/* array: 1 elements */
		"Translate to", noIcon, noKey, noMark, italic
	}
};


resource 'ics8' (kSHLSaCDocumentIconID) {
	$"00FF FFFF FFFF FFFF FFFF FF00 0000 0000"
	$"00FF F5F5 F5F5 F5F5 F5F5 FFFF 0000 0000"
	$"00FF F5F5 F5F5 F5F5 F5F5 FF2B FF00 0000"
	$"00FF F5F5 F5F5 F5F5 F5F5 FFFF FFFF 0000"
	$"00FF F5F5 F5F5 F5F5 F5F5 F5F5 F5FF 0000"
	$"00FF F5F5 F5F5 F5F5 F5F5 F5F5 F5FF 0000"
	$"00FF F5F5 FFF5 FFFF F5FF FFF5 F5FF 0000"
	$"00FF F5F5 F5F5 F5F5 F5F5 F5F5 F5FF 0000"
	$"00FF F5F5 FFFF FFFF F5FF FFF5 F5FF 0000"
	$"00FF F5F5 F5F5 F5F5 F5F5 F5F5 F5FF 0000"
	$"00FF F5F5 FFFF F5FF FFF5 FFF5 F5FF 0000"
	$"00FF F5F5 F5F5 F5F5 F5F5 F5F5 F5FF 0000"
	$"00FF F5F5 F5F5 F5F5 F5F5 F5F5 F5FF 0000"
	$"00FF F5F5 F5F5 F5F5 F5F5 F5F5 F5FF 0000"
	$"00FF F5F5 F5F5 F5F5 F5F5 F5F5 F5FF 0000"
	$"00FF FFFF FFFF FFFF FFFF FFFF FFFF"
};

resource 'ics8' (kSHLSaCStationeryIconID) {
	$"00FF FFFF FFFF FFFF FFFF FFFF 0000 0000"
	$"00FF F5F5 F5F5 F5F5 F5F5 F5FF 0000 0000"
	$"00FF F5F5 F5F5 F5F5 F5F5 F5FF FFFF 0000"
	$"00FF F5F5 F5F5 F5F5 F5F5 F5FF F9FF 0000"
	$"00FF F5F5 FFF5 FFFF F5FF F5FF F9FF 0000"
	$"00FF F5F5 F5F5 F5F5 F5F5 F5FF F9FF 0000"
	$"00FF F5F5 FFFF FFFF F5FF F5FF F9FF 0000"
	$"00FF F5F5 F5F5 F5F5 F5F5 F5FF F9FF 0000"
	$"00FF F5F5 FFFF F5FF FFF5 F5FF F9FF 0000"
	$"00FF F5F5 F5F5 F5F5 F5F5 F5FF F9FF 0000"
	$"00FF F5F5 F5F5 F5F5 FFFF FFFF F9FF 0000"
	$"00FF F5F5 F5F5 F5F5 FFF6 FFF9 F9FF 0000"
	$"00FF F5F5 F5F5 F5F5 FFFF F9F9 F6FF 0000"
	$"00FF FFFF FFFF FFFF FFF9 F9F6 F6FF 0000"
	$"0000 00FF F9F9 F9F9 F9F9 F6F6 F6FF 0000"
	$"0000 00FF FFFF FFFF FFFF FFFF FFFF"
};

resource 'ics4' (kSHLSaCDocumentIconID) {
	$"0FFF FFFF FFF0 0000 0F00 0000 00FF 0000"
	$"0F00 0000 00FC F000 0F00 0000 00FF FF00"
	$"0F00 0000 0000 0F00 0F00 0000 0000 0F00"
	$"0F00 F0FF 0FF0 0F00 0F00 0000 0000 0F00"
	$"0F00 FFFF 0FF0 0F00 0F00 0000 0000 0F00"
	$"0F00 FF0F F0F0 0F00 0F00 0000 0000 0F00"
	$"0F00 0000 0000 0F00 0F00 0000 0000 0F00"
	$"0F00 0000 0000 0F00 0FFF FFFF FFFF FF"
};

resource 'ics4' (kSHLSaCStationeryIconID) {
	$"0FFF FFFF FFFF 0000 0F00 0000 000F 0000"
	$"0F00 0000 000F FF00 0F00 0000 000F DF00"
	$"0F00 F0FF 0F0F DF00 0F00 0000 000F DF00"
	$"0F00 FFFF 0F0F DF00 0F00 0000 000F DF00"
	$"0F00 FF0F F00F DF00 0F00 0000 000F DF00"
	$"0F00 0000 FFFF DF00 0F00 0000 FCFD DF00"
	$"0F00 0000 FFDD CF00 0FFF FFFF FDDC CF00"
	$"000F DDDD DDCC CF00 000F FFFF FFFF FF"
};

resource 'ics#' (kSHLSaCDocumentIconID) {
	{	/* array: 2 elements */
		/* [1] */
		$"7FE0 4030 4028 403C 4004 4004 4B64 4004"
		$"4F64 4004 4DA4 4004 4004 4004 4004 7FFC",
		/* [2] */
		$"7FE0 7FF0 7FF8 7FFC 7FFC 7FFC 7FFC 7FFC"
		$"7FFC 7FFC 7FFC 7FFC 7FFC 7FFC 7FFC 7FFC"
	}
};

resource 'ics#' (kSHLSaCStationeryIconID) {
	{	/* array: 2 elements */
		/* [1] */
		$"7FF0 4010 401C 4014 4B54 4014 4F54 4014"
		$"4D94 4014 40F4 40A4 40C4 7F84 1004 1FFC",
		/* [2] */
		$"7FF0 7FF0 7FFC 7FFC 7FFC 7FFC 7FFC 7FFC"
		$"7FFC 7FFC 7FFC 7FFC 7FFC 7FFC 1FFC 1FFC"
	}
};



//-------------------------------------------------------------------------------------
// Folder names
//-------------------------------------------------------------------------------------


resource 'STR ' (kODEditorsFldrStrID
#If ODDebug
, "Editors Folder name"
#endif
) {
	"Editors"
};

resource 'STR ' (kODStationeryFldrStrID
#If ODDebug
, "Stationery Folder name"
#endif
) {
	"Stationery"
};

resource 'STR ' (kODOpenDocPartsFolderStrID
#If ODDebug
, "OpenDoc Parts Folder name"
#endif
) {
	"OpenDoc"
};

resource 'STR ' (kODShellPlugInFolderStrID
#If ODDebug
, "OD Shell Plug Ins Folder name"
#endif
) {
	"OpenDoc Shell Plug-Ins"
};

// Note: the following two resources were copied from Core.r.

resource 'STR ' (kSHLPrefsFileNameResID
#If ODDebug
, "ODPrefs file name"
#endif
) {
	"OpenDoc Preferences"
};

resource 'STR ' (kSHLPrefsFolderNameResID
#If ODDebug
, "ODPrefs folder name"
#endif
) {
	"OpenDoc Prefs"
};

resource 'STR ' (kSHLLauncherNameResID
#if ODDebug
, "Launcher name"
#endif
) {
	"OpenDoc"
};

resource 'STR ' (kSHLSystemProcessNameResID
#if ODDebug
, "System Process name"
#endif
) {
	"OpenDoc"
};

resource 'STR ' (kSHLOldLauncherNameResID
#if ODDebug
, "Name to use for obsolete launcher"
#endif
) {
	"OpenDoc 1.0 Document Launcher"
};

//-------------------------------------------------------------------------------------
// New filename construction template
//-------------------------------------------------------------------------------------

// Shell new filename construction template
// ^0 is the text seed (like "untitled"), ^1 is the date (like "5/18/95")
// Localizers can change string delimiter & order, and can add suffix or prefix
// Note that string will be truncated from end when a number is added to achieve
// filename uniqueness.

resource 'STR ' (kSHLNewFilenameTemplateStrID) {
	"^0 ^1"
};

#include "UtilCmmn.r"

//-------------------------------------------------------------------------------------
// Save, Close and Delete menuitem text construction templates
//-------------------------------------------------------------------------------------

resource 'STR ' (kSHLSaveItemTemplateStrID) {
	"Save ^0"
};

resource 'STR ' (kSHLCloseItemTemplateStrID) {
	"Close ^0"
};

resource 'STR ' (kSHLDeleteItemTemplateStrID) {
	"Delete ^0"
};

resource 'STR ' (kSHLPlugInErrorStrID, "") {
	"The OpenDoc Shell Plug-Ins ^0 failed to install properly due to the "
	"following error: ^1 Try removing it from your Shell Plug-Ins folder."
};


//-------------------------------------------------------------------------------------
// Error handling resources
//-------------------------------------------------------------------------------------

/*
	OpenDoc Error Resources
	based on MacApp ErrorMgr resources
*/

/* NOTE:

	By Toolbox convention, the OK button is item 1 and the Cancel button
		is item 2.
*/

resource 'ALRT' (kSHLphGenError,purgeable) {
	{80, 80, 258, 462},
	kSHLphGenError,
	{	/* array: 4 elements */
		/* [1] */
		OK, visible, sound1;
		/* [2] */
		OK, visible, sound1;
		/* [3] */
		OK, visible, sound1;
		/* [4] */
		OK, visible, sound1
	}
#if SystemSevenOrLater
	,
	alertPositionParentWindowScreen
#endif
};

resource 'DITL' (kSHLphGenError,purgeable) {
	{	/* array DITLarray: 3 elements */
		/* [1] */
		{140, 258, 158, 332},
		Button {
			enabled,
			"OK"
		},
		/* [2] */
		/*	^0 = userfriendly reason for error and how to recover; 
				e.g., 'The disk is locked.  Eject the disk and move the tab.'
			or
				technically detailed reason for error
				e.g., 'Volume error.'
		*/
		{10, 70, 135, 332},
		StaticText {
			disabled,
			"^0\n^2"		/* ^2 is the exception message str in the technical err alert. */
		},
		/* [3] */
		/*	^1 = exception number for error; 
				e.g., 'Exception: 5'
		*/
		{140, 70, 158, 250},
		StaticText {
			disabled,
			"^1"
		},
		/* [4] */
		{10, 20, 42, 52},
		Icon {
			disabled,
			0
		}
	}
};

//-------------------------------------------------------------------------------------
// Misc. Shell Strings
//-------------------------------------------------------------------------------------


resource 'STR#' (kSHLStrsID, purgeable) {
	{
		 "" /* Unused slot. May be reused. See ShellDef.h */
/*		;"Close"				*/
/*		;"Close Document"		*/
		;"Part Info"
		
		;"Save a copy as:"
		
		;"Exception: "
		
		;"Save as:"
		
	}
};

resource 'STR#' (kSHLMiscErrorStringsStrID, purgeable) {
	{
/* 1 */	"This version of OpenDoc cannot run under MacOS 8. Please install"
			" a newer version."
	}
};

resource 'STR ' (kSHLCopyDefaultStrID, purgeable) {
	"^0 copy"
};

resource 'STR ' (kSHLPreODDocNameStrID, purgeable) {
	"^0 OD"
};



/********* Stuff for the Debug menu **********/

#if ODDebug

resource 'ALRT' (980
#If ODDebug
, "Heap Info [debug only]"
#endif
) {
	{40, 40, 201, 412},
	980,
	{	/* array: 4 elements */
		/* [1] */
		OK, visible, silent,
		/* [2] */
		OK, visible, silent,
		/* [3] */
		OK, visible, silent,
		/* [4] */
		OK, visible, silent
	},
	alertPositionParentWindowScreen
};


resource 'DITL' (980
#If ODDebug
, "Heap Info [debug only]"
#endif
) {
	{	/* array DITLarray: 7 elements */
		/* [1] */
		{132, 296, 152, 354},
		Button {
			enabled,
			"OK"
		},
		/* [2] */
		{10, 72, 29, 360},
		StaticText {
			disabled,
			"^0:"
		},
		/* [3] */
		{28, 72, 79, 162},
		StaticText {
			disabled,
			"Bytes used:\nBytes free:\nBlock count:"
		},
		/* [4] */
		{28, 161, 79, 280},
		StaticText {
			disabled,
			"^1"
		},
		/* [5] */
		{89, 72, 108, 360},
		StaticText {
			disabled,
			"^2:"
		},
		/* [6] */
		{107, 72, 158, 162},
		StaticText {
			disabled,
			"Bytes used:\nBytes free:\nLargest free space:"
		},
		/* [7] */
		{107, 161, 158, 280},
		StaticText {
			disabled,
			"^3"
		}
	}
};

#endif /* ODDebug */

resource 'ALRT' (493
#if ODDebug
,"Shell Low Memory Error"
#endif
/* not purgable because lowmem*/,preload) {
	{40, 40, 142, 318},
	493,
	{	/* array: 4 elements */
		/* [1] */
		OK, visible, sound1,
		/* [2] */
		OK, visible, sound1,
		/* [3] */
		OK, visible, sound1,
		/* [4] */
		OK, visible, sound1
	},
	alertPositionMainScreen
};

/* NOTE THAT WE EXPLICITLY USE ICON #0 IN THIS DITL RATHER THAN CALLING
	StopAlert TO INSERT THE ICON. WE DO THIS BECAUSE WE WANT TO MAKE SURE
	THE ICON IS ALWAYS LOADED INTO MEMORY */

resource 'DITL' (493
#if ODDebug
,"Shell Low Memory Error"
#endif
/* not purgable because lowmem*/,preload) {
	{	/* array DITLarray: 3 elements */
		/* [1] */
		{69, 201, 89, 263},
		Button {
			enabled,
			"Quit"
		},
		/* [2] */
		{10, 72, 58, 265},
		StaticText {
			disabled,
			"Sorry, this document must immediately qu"
			"it due to very low memory."
		},
		/* [3] */
		{11, 20, 43, 52},
		Icon {
			disabled,
			0
		}
	}
};

resource 'ALRT' (496
#if ODDebug
,"Shell Generic Goodbye Error"
#endif
, purgeable /*<eeh> added for 1384356*/,preload) {
	{40, 40, 142, 318},
	496,
	{	/* array: 4 elements */
		/* [1] */
		OK, visible, sound1,
		/* [2] */
		OK, visible, sound1,
		/* [3] */
		OK, visible, sound1,
		/* [4] */
		OK, visible, sound1
	},
	alertPositionMainScreen
};

/* NOTE THAT WE EXPLICITLY USE ICON #0 IN THIS DITL RATHER THAN CALLING
	StopAlert TO INSERT THE ICON. WE DO THIS BECAUSE WE WANT TO MAKE SURE
	THE ICON IS ALWAYS LOADED INTO MEMORY */

resource 'DITL' (496
#if ODDebug
,"Shell Generic Goodbye Error"
#endif
/* not purgable because lowmem*/,preload) {
	{	/* array DITLarray: 3 elements */
		/* [1] */
		{69, 201, 89, 263},
		Button {
			enabled,
			"Quit"
		},
		/* [2] */
		{10, 72, 58, 265},
		StaticText {
			disabled,
			"Sorry, an irrecoverable error has occurred opening this document."
		},
		/* [3] */
		{11, 20, 43, 52},
		Icon {
			disabled,
			0
		}
	}
};

resource 'ALRT' (497
#if ODDebug
,"Shell Corrupt Doc Error"
#endif
, purgeable /*<eeh> added for 1384356*/,preload) {
	{40, 40, 142, 318},
	497,
	{	/* array: 4 elements */
		/* [1] */
		OK, visible, sound1,
		/* [2] */
		OK, visible, sound1,
		/* [3] */
		OK, visible, sound1,
		/* [4] */
		OK, visible, sound1
	},
	alertPositionMainScreen
};

/* NOTE THAT WE EXPLICITLY USE ICON #0 IN THIS DITL RATHER THAN CALLING
	StopAlert TO INSERT THE ICON. WE DO THIS BECAUSE WE WANT TO MAKE SURE
	THE ICON IS ALWAYS LOADED INTO MEMORY */

resource 'DITL' (497
#if ODDebug
,"Shell Corrupt Doc Error"
#endif
, purgeable /*<eeh> added for 1384356*/,preload) {
	{	/* array DITLarray: 3 elements */
		/* [1] */
		{69, 201, 89, 263},
		Button {
			enabled,
			"Quit"
		},
		/* [2] */
		{10, 72, 58, 265},
		StaticText {
			disabled,
			"Sorry, this document is corrupt and cannot be repaired."
		},
		/* [3] */
		{11, 20, 43, 52},
		Icon {
			disabled,
			0
		}
	}
};

resource 'ALRT' (495
#if ODDebug
,"Finder not running"
#endif
, purgeable /*<eeh> added for 1384356*/) {
 	{40, 40, 168, 390},
	495,
	{	/* array: 4 elements */
		/* [1] */
		OK, visible, sound1,
		/* [2] */
		OK, visible, sound1,
		/* [3] */
		OK, visible, sound1,
		/* [4] */
		OK, visible, sound1
	},
	alertPositionMainScreen
};

resource 'DITL' (495
#if ODDebug
,"Finder not running"
#endif
, purgeable /*<eeh> added for 1384356*/) {
	{	/* array DITLarray: 2 elements */
		/* [1] */
		{91, 270, 111, 328},
		Button {
			enabled,
			"OK"
		},
		/* [2] */
		{10, 83, 76, 338},
		StaticText {
			disabled,
			"This OpenDoc command cannot be completed"
			" because the Finder is not running. To s"
			"tart the Finder, close all documents and"
			" quit all applications."
		}
	}
};

// THIS ALERT AND DITL ARE DUPLICATED IN OPENDOCX.RES

resource 'ALRT' (2001, purgeable /*<eeh> added for 1384356*/) {
	{40, 40, 141, 389},
	2001,
	{	/* array: 4 elements */
		/* [1] */
		OK, visible, sound1,
		/* [2] */
		OK, visible, sound1,
		/* [3] */
		OK, visible, sound1,
		/* [4] */
		OK, visible, sound1
	},
	centerParentWindowScreen
};

resource 'DITL' (2001, purgeable /*<eeh> added for 1384356*/) {
	{	/* array DITLarray: 4 elements */
		/* [1] */
		{68, 278, 88, 336},
		Button {
			enabled,
			"OK"
		},
		/* [2] */
		{10, 64, 59, 339},
		StaticText {
			disabled,
			"^0"
		}
	}
};

/* Can't create stationery */

resource 'ALRT' (456
#if ODDebug
,"Can't create stationery"
#endif
, purgeable) {
	{40, 40, 142, 318},
	456,
	{	/* array: 4 elements */
		/* [1] */
		OK, visible, sound1,
		/* [2] */
		OK, visible, sound1,
		/* [3] */
		OK, visible, sound1,
		/* [4] */
		OK, visible, sound1
	},
	alertPositionMainScreen
};

resource 'DITL' (456
#if ODDebug
,"Can't create stationery"
#endif
, purgeable) {
	{	/* array DITLarray: 3 elements */
		/* [1] */
		{69, 201, 89, 263},
		Button {
			enabled,
			"OK"
		},
		/* [2] */
		{10, 72, 58, 265},
		StaticText {
			disabled,
			"Sorry, stationery cannot be created for "
			"viewers."
		},
		/* [3] */
		{11, 20, 43, 52},
		Icon {
			disabled,
			0
		}
	}
};



/*#endif*/
