/*
	File:		ShellDef.h

	Contains:	All non-local definitions for the "shell," meaning not
	only the DocShell, but also DraftWindow and the launcher.  The impetus
	for combining previously separate files is to avoid resource id conflicts
	within the shell subsystem and between it an the rest of OpenDoc
	
	Merged from: DfWinDef.h, ODXDef.h, and RlShlDef.h

	Owned by: Nick Pilch

	Copyright:	 1994 - 1997 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		<24>	 1/13/97	DH		Minimum and default document heap size
									increased to 384K.
		<23>	  1/8/97	DH		1401434: New strings for System process to
									alert user that document is missing.
		<22>	  12/18/96	DH		1375780,1612023: Added new dialog constants
									and reordered many others.
		<21>	  10/22/96	DH		1336808,1333949,1368888,1375780
									Added/changed constants for low mem dialog
									and alert resources.
		<20>	02.10.1996	NP		1391011: hard-coded strings, and general
									clean-up.
		<19>	27.09.1996	NP		1385775: Incorrect dialog after OD
									initialization fails.
		<18>	11.09.1996	NP		1386084: constant for earliest compatible
									system version
		<17>	  9/4/96	EL		1306385: Remove user item that are used as
									default button in dialog.
		<16>	 7/30/96	eeh		1372943: kODAlertShellGenericError and
									kODAlertShellCorruptDocError
		<15>	17.07.1996	NP		1369268: Change app heap size.
		<14>	 6/27/96	TJ		Removed editor type, now using CFMtype for
									system headers.
		<13>	 6/23/96	NP		10002: Launch time speedups.
		<12>	 6/21/96	jpa		T10002: Changed min/default heap size to
									210k. 1358818: Added rsrc ID for File menu.
		<11>	 6/21/96	eeh		task 10008: add buttons etc. for AppleGuide
		<10>	 6/19/96	NP		10002: Launch time speedups.
		 <9>	 6/18/96	NP		10002: Launch time speedups.
		 <8>	 6/14/96	NP		10002: Launch time speedups
		 <7>	.06.1996	NP		10002: Launch time fixes, system process.
		 <6>	 5/31/96	jpa		T10012: Added Quit menu cmd for CyberDog
									(does not appear in regular docs!)
		 <5>	.04.1996	NP		1286751: Present alert when trying to use
									Open when no Finder is running.
		 <4>	 4/15/96	TJ		Nolonger include ODXDef.h
		 <3>	 3/29/96	DM		1334273: rsrc const for new alert 493
		 <2>	.03.1996	NP		1307182: Removed STR# index constant.
		 <4>	10/31/95	TJ		Made the Splash Screen Non-Modal.#1297413
		 <3>	10/26/95	T		1296574 GM:Cannot set default memory
									partition size.
		 <2>	10/24/95	jpa		1293441: Added more SpaceIsLow alerts.
		 <1>	 9/29/95	eeh		1287262: first checked in
	
	In Progress:
		
		NP: We only have to have unique resource numbers for resources of the
		same kind. Currently, every resource is given a new unique number here.
		We are going to run out of numbers in our range if we don't change the
		scheme.

		LDEFs can have the same ID as anything, since having that ID means
		nothing.  (That is, there's no assumption in the code or toolbox like
		the one that DLOG and DITL ids will match.)
		
*/

#ifndef _SHELLDEF_
#define _SHELLDEF_

#ifndef _RSRCBASE_
#include "RsrcBase.h"
#endif


#define SUPPORT_QUIT	/* Define to add Quit cmd for APPL processes only, *not* documents */

#ifdef SUPPORT_QUIT
#define kODCommandQuit	(kODCommandPrint + 1)	/*//jpa $$$$$ Should be in CmdDef.idl*/
/* CyberDog uses the value of this constant so don't mess with it unnecessarily */
#endif


#define kODShellLDEFRsrcBase	kODShellRsrcBase

/*****************************************************************************
	From:		RlShlDef.h
	The constants following are for the DocShell itself
*****************************************************************************/

//==============================================================================
// Resources
//==============================================================================

//------------------------------------------------------------------------------
// MENU
//------------------------------------------------------------------------------

// File menu to use instead of Document menu in 'APPL' process:
#define kODFileMenuID					(kODShellRsrcBase + 0)

#define kSHLSaCKindPopupMenu      		(kODShellRsrcBase + 14)

//------------------------------------------------------------------------------
// ALRT, DLOG and associated DITL
//------------------------------------------------------------------------------

/*The Splash Screen Dialog ID */
#define kODSplashDlogID					131

// Do you want to save changes before closing? DLOG
#define	kSHLsvChanges					(kODShellRsrcBase + 0)				

// Revert to the last saved version? DLOG
#define	kSHLsvRevert					(kODShellRsrcBase + 1)					

// Delete this document? DLOG
#define	kSHLDeleteDLOG					(kODShellRsrcBase + 2)					

// Error Handling ALRT
#define	kSHLphGenError					(kODShellRsrcBase + 3)

// Low-memory ALRTs:
// This next two alerts also used for the R/O case.
#define	kSHLphTempMemIsLow				(kODShellRsrcBase + 7)
#define	kSHLphTempMemIsLowFatal			(kODShellRsrcBase + 9)

#define	kSHLphAppSpaceIsLow				(kODShellRsrcBase + 10)
#define	kSHLphAppSpaceIsLowRO			(kODShellRsrcBase + 15)
#define	kSHLphAppSpaceIsLowFatal		(kODShellRsrcBase + 14) 
#define	kSHLphAppSpaceIsLowFatalRO		(kODShellRsrcBase + 13) 

#define	kSHLphDocSizeChange				(kODShellRsrcBase + 16) 
#define	kSHLphDocSizeChangeRO			(kODShellRsrcBase + 17) 

// Low memory dialogs
#define	kSHLphAppSpaceIsLowDlg			(kODShellRsrcBase + 12)

#define kSHLAlertCantMakeStationery		(kODShellRsrcBase + 6)

// Save A Copy dialog
#define kSHLSaveCopyDlgID				(kODShellRsrcBase + 11)

#define kSHLSaCDocumentRadioBtn   		13
#define kSHLSaCStationeryRadioBtn 		14
#define kSHLSaCKindPopupItem      		15
#define kSHLSaCDocumentIconItem   		16
#define kSHLSaCStationeryIconItem 		17

// Save To Different Volume dialog
#define kSHLSaveDiffVolDialogID			(kODShellRsrcBase + 27)

#define	kSHLAboutBoxDlogID				(kODShellRsrcBase + 28)

// ALRT resource IDs
#define kODXErrAlert 					(kODShellRsrcBase + 29)
#define	kODXAnonErrAlert				(kODShellRsrcBase + 30)

#define kDraftsDlgID					(kODShellRsrcBase + 34)

/* fields in Drafts dialog */
#define kDraftsCreateBtn			1
#define kDraftsDoneBtn				2
#define kDraftsDeleteBtn			3
#define kDraftsOpenBtn				4
#define kDraftsCreatorStaticTxt		5
#define kDraftsDraftStaticTxt		6
#define kDraftsCreatedStaticTxt		7
#define kDraftsArrowStaticTxt		8			/* a null string */
#define kDraftsCommentStaticTxt		9
#define kDraftsHeaderRectUserItem	10
#define kDraftsHorizRectUserItem	11
#define kDraftsListUserItem			12
#define kDraftsAGButtonItem			13

#define kCreateDraftsDlgID				(kODShellRsrcBase + 35)

/* fields in Create Drafts dialog */
#define kCreateDraftsCreateBtn			1
#define kCreateDraftsCancelBtn			2
#define kCreateDraftsDraftStaticTxt		3
#define kCreateDraftsNameStaticTxt		4
#define kCreateDraftsNameEditTxt		5
#define kCreateDraftsCommentsStaticTxt	6
#define kCreateDraftsCommentsEditTxt	7
#define kCreateDraftsDraftEditTxt		8
#define kCreateDraftsAGButtonItem		9

#define kDeleteDraftsDlogID				(kODShellRsrcBase + 36)

/* fields in Delete Drafts dialog */
#define kDeleteDraftsDeleteBtn			1
#define kDeleteDraftsCancelBtn			2
#define kDeleteDraftsStaticTxt			3

#define kODAlertShellLowMemoryError		(kODShellRsrcBase + 43)

#define	kODXDoubleClickAlert			(kODShellRsrcBase + 44)

#define kODAlertShellFinderNotRunning	(kODShellRsrcBase + 45)

#define kODAlertShellGenericError		(kODShellRsrcBase + 46)
#define kODAlertShellCorruptDocError	(kODShellRsrcBase + 47)	

//------------------------------------------------------------------------------
// errs
//------------------------------------------------------------------------------

// Error Handling errs & STR#
#define	kODErrUserID					(kODShellRsrcBase + 6)
#define	kODErrSupportID					(kODShellRsrcBase + 7)

//------------------------------------------------------------------------------
// STR#
//------------------------------------------------------------------------------

#define kODSysProcStringsID				(kODShellRsrcBase + 0) 
// not used here. See SPDefs.h. Put here in order to reserve this number
//	in the shell's range.

#define kODSPStrIndexComplexErrMssg		1
#define kODSPStrIndexTwoPartErrMssg		2
#define kODSPStrIndexOnePartErrMssg		3
#define kODSPStrIndexWrongVersSys		4
#define kODSPStrIndexNoMemoryToStartUp	5
#define kODSPStrIndexUnrecoverableStartUpError	6
#define kODSPStrIndexCouldntRelaunchDoc	7
#define kODSPStrIndexMissingDocument	8

//#define kODSPStrIndexVerySimpleErrMssg	xx

#define	kODErrUserIDStrings				(kODShellRsrcBase + 8)
#define	kODErrSupportIDStrings			(kODShellRsrcBase + 9)

// Dragged-to-Trash alert [DISABLED]
// #define kSHLDragToTrashAlert			(kODShellRsrcBase + 10)

// Shell Strings STR#
#define kSHLStrsID						(kODShellRsrcBase + 19)
//#define kSHLStrIndAboutOD				1
// #define kSHLStrIndMClose				2
// #define kSHLStrIndMCloseDoc			3
#define kSHLStrIndMPartInfo				2
#define kSHLStrIndDSaveACopy			3
#define kSHLStrIndDException			4
#define kSHLStrIndDSaveAs				5

// Strings for the Launcher
#define	kODXLibsFldrStrID				(kODShellRsrcBase + 32)
#define kODXLibsFldrStrIndex			1
#define kODXTriedToOpenPrefsStrIndex	2

#define kLaunchFailedErrStrings			(kODShellRsrcBase + 33)

// STR#-misc errors
#define kSHLMiscErrorStringsStrID		(kODShellRsrcBase + 48)
#define kSHLCannotRunUnderMacOS8Index	1

// Doc size change strings
#define kSHLDecreaseSizeSTR				(kODShellRsrcBase + 27)
#define kSHLIncreaseSizeSTR				(kODShellRsrcBase + 28)

//------------------------------------------------------------------------------
// STR
//------------------------------------------------------------------------------

#define kODSPStrCannotOpenRODocSizeErrMsg	(kODShellRsrcBase + 0) 
// not used here. See SPDefs.h. Put here in order to reserve this number
//	in the shell's range.

#define kSHLsvChangesCmdKeyStrs			kSHLsvChanges
#define kSHLsvRevertCmdKeyStrs			kSHLsvRevert
#define kSHLDeleteDLOGCmdKeyStrs		kSHLDeleteDLOG

// Folder Name strings searching
#define kODEditorsFldrStrID				(kODShellRsrcBase + 15)
#define kODOpenDocPartsFolderStrID		(kODShellRsrcBase + 16)
#define kODShellPlugInFolderStrID		(kODShellRsrcBase + 17)
#define kODStationeryFldrStrID			(kODShellRsrcBase + 18)

// Shell default save a copy string
#define kSHLCopyDefaultStrID			(kODShellRsrcBase + 20)					

// Shell opening pre-OD document name string
#define kSHLPreODDocNameStrID			(kODShellRsrcBase + 21)

// Shell new filename construction template
#define kSHLNewFilenameTemplateStrID	(kODShellRsrcBase + 22)

// Shell menuitem text construction templates
#define kSHLSaveItemTemplateStrID		(kODShellRsrcBase + 23)
#define kSHLCloseItemTemplateStrID		(kODShellRsrcBase + 24)
#define kSHLDeleteItemTemplateStrID		(kODShellRsrcBase + 25)

#define kSHLPlugInErrorStrID			(kODShellRsrcBase + 26)

// Prefs file name resource ID
#define kSHLPrefsFileNameResID			(kODShellRsrcBase + 31)

#define kDraftsWnTitleResID				(kODShellRsrcBase + 37)
#define kDraftsWnDateSpaceResID			(kODShellRsrcBase + 38)

#define kDraftsCmdKeyStrs				(kODShellRsrcBase + 39)
#define kDraftsCreateCmdKeyStrs			(kODShellRsrcBase + 40)

#define kSHLPrefsFolderNameResID		(kODShellRsrcBase + 43)
#define kSHLLauncherNameResID			(kODShellRsrcBase + 46)
#define kSHLSystemProcessNameResID		(kODShellRsrcBase + 47)
#define kSHLOldLauncherNameResID		(kODShellRsrcBase + 49)

//------------------------------------------------------------------------------
// ICON
//------------------------------------------------------------------------------

// Save A Copy dialog
#define kSHLSaCDocumentIconID	   		(kODShellRsrcBase + 12)
#define kSHLSaCStationeryIconID   		(kODShellRsrcBase + 13)

//------------------------------------------------------------------------------
// ics#, ics4, ics8
//------------------------------------------------------------------------------

#define	kDWClosedTriangleResID		(kODShellRsrcBase + 41)
#define	kDWOpenTriangleResID		(kODShellRsrcBase + 42)

//------------------------------------------------------------------------------
// LDEF
//------------------------------------------------------------------------------

#define kDraftWinLDEFID			(kODShellLDEFRsrcBase + 0)		
#define kDraftWinLDEFAddrOffset	0x00000008
#define kDWNumColumns 4

//==============================================================================
// Error Codes
//==============================================================================

// Private error codes. Before adding more, check ErrorDef.idl to ensure uniqueness.
#define kODErrAllWritableVolumesAreFull   		-29826
#define kODErrVolumesIsFull   					-29825
#define kODErrDirectoryNotHaveWriteAccess  		-29824
#define kODErrCannotSaveACopyOntoCurrentDoc 	-29823
#define kODErrCannotSaveACopyOntoBusyDoc  		-29822
#define kODErrSystemVolumeIsFull   				-29821
#define kODErrCouldNotBindForeignDocument		-29820
#define kODErrCannotFindShellPlugInsFolder		-29819
#define kODErrCannotFindSystemEditorsFolder		-29818
#define kODErrCannotFindOpenDocLibrariesFolder	-29817
#define	kODErrEditorLibraryNotFound				-29816
#define kODErrCannotSaveAsOntoBusyDoc			-29815

//==============================================================================
// Resource types and IDs
//==============================================================================

#define kPrefFileType		'pref'
// for stationery creation
#define	kDocSeedResType		'seed'
#define	kDocSeedResID		1

// Document Stub Data					Document Stub App
// kDocStubRsrcType kDocStubRsrcID0		kcfrgRsrcType	kcfrgID
// kDocStubRsrcType kDocStubRsrcID1		kMainCFMType	kMainCFMPPCID
// kDocStubRsrcType kDocStubRsrcID2		kscszRsrcType	kscszID
// kDocStubRsrcType kDocStubRsrcID3		kMainCFMType 	kMainCFM68KID
// kDocStubRsrcType kDocStubRsrcID4		kMainCFMType 	kMainCFM68KID1
// kDocStubRsrcType kDocStubRsrcID5		kCODERsrcType	kCODEID0
// kDocStubRsrcType kDocStubRsrcID6		kCODERsrcType 	kCODEID1
// kDocStubRsrcType kDocStubRsrcID7		kCODERsrcType	kCODEID6
// kDocStubRsrcType kDocStubRsrcID8		kCODERsrcType 	kCODEID7
// kDocStubRsrcType kDocStubRsrcID9		kMainCFMType 	kSOMMemCFM68KID
// kDocStubRsrcType kDocStubRsrcID10	kMainCFMType 	kSOMMemCFMPPCID
// kSIZERsrcType 	kDocStubSIZEID		kSIZERsrcType	kSIZEID
//

// for document stub data (e.g. types & ids of the doc stub app resources when stored
// in OpenDoc & OpenDoc Shell)
#define	kSIZERsrcType		'SIZE'
#define kDocStubSIZEID		3

// Default SIZE ID for use if there is no prefs file:
#define kDocStubDefaultSIZEID	4

#define kDocStubRsrcType	'DSDT'	// for Document Stub DaTa, i.e. how these resources
									// are actually stored in OpenDoc & OpenDoc Shell
									// except for the SIZE resource which we keep as
									// a SIZE resource for the convenience of editing it
									// in Resedit.
									
#define kDocStubRsrcID0		0
#define kDocStubRsrcID1		1
#define kDocStubRsrcID2		2
#define kDocStubRsrcID3		3
#define	kDocStubRsrcID4		4
#define	kDocStubRsrcID5		5
#define	kDocStubRsrcID6		6
#define	kDocStubRsrcID7		7
#define	kDocStubRsrcID8		8
#define	kDocStubRsrcID9		9
//#define	kDocStubRsrcID10	10

// for document stub construction
#define	kMainCFMType		'rseg'  // used to be 'DSTB' for Document STuB
									// but CFM68K uses 'rseg'.
#define	kMainCFM68KID		0
#define	kMainCFM68KID1		1
#define	kMainCFMPPCID		2
#define kSOMMemCFM68KID		3
#define kSOMMemCFMPPCID		4

#define kcfrgRsrcType		'cfrg'
#define kcfrgID				0
#define	kscszRsrcType		'scsz'
#define kscszID				0

#define kDocStubStackSize	(64 * 1024)		// 64K stack size

// 1242449 Should probably create a common def.h constant kDocStubMinHeapSize
// accessible from both subsystems. NP: Something like ODDefs.h.
#define	kDocStubMinHeapSize	(384 * 1024)	// min heap size;was 320K (Changed 1/13/97 - dh)
#define	kDocStubDefaultSize	(384 * 1024)	// default app heap; was 256K 

#define kCODERsrcType		'CODE'
#define kCODEID0			0
#define kCODEID1			1
#define kCODEID6			6
//#define kCODEID7			7

#define	kSIZEID				-1

#define kODSystemProcessSignature 'odbg'
#define kODDisabledProcessSignature 'odno'

//==============================================================================
// Apple event constants
//==============================================================================

#define kODReopenDocEventID 'clre'
#define kODFSSpecKeyword
#define kODProcessSerialNumber

//==============================================================================
// Misc
//==============================================================================

// Keep these two in sync!
#define kOD1pt2EarliestCompatibleSysVersNum 0x753
#define kOD1pt2EarliestCompatibleSysVersStr "7.5.3"


#endif	/* _SHELLDEF_ */

